### Multinomial Logit model estimation

### データファイルの読み込み
Data <- read.csv("Final7.csv",header=T,fileEncoding="UTF-8")

## データ数:Dataの行数を数える
hh <- nrow(Data)

## パラメータの初期値の設定
b0 <- numeric(7)

##### C Logit model の対数尤度関数の定義 #####

fr <- function(x) {
  ### パラメータの宣言：
  ## 駐輪場利用(legal-irregular)
  ## 不定期利用料金
  b1 <- x[1]
  ## 駐車場収容台数
  b2 <- x[2]
  
  ## 駐輪場利用(legal-regular)
  ## 定期利用料金
  b3 <- x[3]
  ## 定数項
  b4 <- x[4]
  
  ## 違法駐輪(illegal)
  ## 不定期効用
  #b6 <- x[6]
  ## 最寄の一時利用可能駐輪場から違法駐輪場所までの距離
  b5 <- x[5]
  ## 心理的要因
  b6 <- x[6]
  ## 定数項
  b7 <- x[7]
  
  ## 合法ダミー
  #d0 <- Data$合法ダミー==0 
  d1 <- Data$low==1 

  
  ## 混雑度有無ダミー設定(混雑度表記があれば1、なければ0)
  #e1 <- Data$congestion_ex_1==1 
  #e2 <- Data$congestion_ex_2==1 
  #e3 <- Data$congestion_ex_3==1 
  #e4 <- Data$congestion_ex_4==1 
  
  ## 選択肢有効性(該当経路が存在すれば1、存在しなければ0)
  #v1 <- Data$validity_1
  #v2 <- Data$validity_2
  #v3 <- Data$validity_3
  #v4 <- Data$validity_4
  
  ## 対数尤度のための変数を宣言
  LL = 0
  
  ## 効用の計算
  ## u1 = 駐輪場利用(不定期)　u2 = 駐輪場利用(定期)  u3 = 違法駐輪

  #u1 <- exp(b1*Data$irregular*Data$irregular_cost + b2*Data$regular*(Data$regular_cost/100)
            #+ b3*Data$capa + b4*Data$distance_legal + b5*matrix(1,nrow=hh,ncol=1))
  
  #u2 <- exp(b6*Data$irregular*Data$irregular_cost + b7*Data$distance_illegal*Data$irregular
            #+ b8*(Data$amount/100))
  
  u1 <- exp(d1*b1*Data$legal_irregular_cost + b2*Data$capa)
  u2 <- exp(d1*b3*Data$legal_regular_cost + b4*matrix(1,nrow=hh,ncol=1))
  u3 <- exp(b5*Data$distance_illegal + b6*Data$amount2 +b7*matrix(1,nrow=hh,ncol=1))
  
  ### 選択確率の計算
  ## 分母となる，各々のexp(V)の和をつくる
  
  deno <- (u1 + u2 +u3)
  
  ## それぞれ計算する
  P1 <- u1 / deno
  P2 <- u2 / deno
  P3 <- u3 / deno
  
  ## 選択確率が0になってしまった場合に起こる問題の回避
  P1 <- (P1!=0)*P1 + (P1==0)
  P2 <- (P2!=0)*P2 + (P2==0)
  P3 <- (P3!=0)*P3 + (P3==0)

  ## 選択結果
  C1 <- Data$select1 == 1
  C2 <- Data$select2 == 1
  C3 <- Data$select3 == 1
  
  ## 対数尤度の計算
  LL <- colSums(C1*log(P1) + C2*log(P2)+ C3*log(P3))
  return(LL)
  
}


##### 対数尤度関数frの最大化#####

##パラメータ値の最適化 

res <- optim(b0,fr, method = "BFGS", hessian = TRUE, control=list(fnscale=-1))
## パラメータ推定値、ヘッセ行列 

b   <- res$par
hhh <- res$hessian

## t値の計算
tval <- b/sqrt(-diag(solve(hhh)))

## 初期尤度
L0 <- fr(b0)
## 最終尤度
LL <- res$value

##### 結果の出力 #####
print(res)
## 初期尤度
print(L0)
## 最終尤度
print(LL)
##ρ^2値
print((L0-LL)/L0)
## 修正済ρ^2値
print((L0-(LL-length(b)))/L0)
##パラメータ推定値 
print(b)
## t値 
print(tval)